@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2009 The PHP Group                                     |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
 
  */
 
-/* $Id: apc_bin.c 284580 2009-07-22 06:05:31Z kalle $ */
+/* $Id: apc_bin.c 307333 2011-01-10 09:02:13Z pajoye $ */
 
 /* Creates a binary architecture specific output to a string or file containing
  * the current cache contents for both fies and user variables.  This is accomplished
@@ -50,7 +50,8 @@ extern int _apc_store(char *strkey, int strkey_len, const zval *val, const uint
             ptr = (void*)((long)(ptr) - (long)(bd)); \
             printf("%x in %s on line %d", ptr, __FILE__, __LINE__); \
         } else if((long)ptr > bd->size) { /* not swizzled */ \
-            apc_eprint("pointer to be swizzled is not within allowed memory range! (%x < %x < %x) in %s on %d", (long)bd, ptr, ((long)bd + bd->size), __FILE__, __LINE__); \
+            apc_error("pointer to be swizzled is not within allowed memory range! (%x < %x < %x) in %s on %d" TSRMLS_CC, (long)bd, ptr, ((long)bd + bd->size), __FILE__, __LINE__); \
+            return; \
         } \
         printf("\n"); \
     } while(0);
@@ -69,7 +70,8 @@ extern int _apc_store(char *strkey, int strkey_len, const zval *val, const uint
         if((long)bd < (long)ptr && (ulong)ptr < ((long)bd + bd->size)) { \
             ptr = (void*)((long)(ptr) - (long)(bd)); \
         } else if((ulong)ptr > bd->size) { /* not swizzled */ \
-            apc_eprint("pointer to be swizzled is not within allowed memory range! (%x < %x < %x) in %s on %d", (long)bd, ptr, ((long)bd + bd->size), __FILE__, __LINE__); \
+            apc_error("pointer to be swizzled is not within allowed memory range! (%x < %x < %x) in %s on %d" TSRMLS_CC, (long)bd, ptr, ((long)bd + bd->size), __FILE__, __LINE__); \
+            return NULL; \
         } \
     } while(0);
 
@@ -81,9 +83,9 @@ extern int _apc_store(char *strkey, int strkey_len, const zval *val, const uint
 #endif
 
 
-static void *apc_bd_alloc(size_t size);
-static void apc_bd_free(void *ptr);
-static void *apc_bd_alloc_ex(void *ptr_new, size_t size);
+static void *apc_bd_alloc(size_t size TSRMLS_DC);
+static void apc_bd_free(void *ptr TSRMLS_DC);
+static void *apc_bd_alloc_ex(void *ptr_new, size_t size TSRMLS_DC);
 
 typedef void (*apc_swizzle_cb_t)(apc_bd_t *bd, zend_llist *ll, void *ptr TSRMLS_DC);
 
@@ -109,18 +111,18 @@ static int apc_unswizzle_bd(apc_bd_t *bd, int flags TSRMLS_DC);
 
 /* {{{ apc_bd_alloc
  *  callback for copy_* functions */
-static void *apc_bd_alloc(size_t size) {
-    return apc_bd_alloc_ex(NULL, size);
+static void *apc_bd_alloc(size_t size TSRMLS_DC) {
+    return apc_bd_alloc_ex(NULL, size TSRMLS_CC);
 } /* }}} */
 
 
 /* {{{ apc_bd_free
  *  callback for copy_* functions */
-static void apc_bd_free(void *ptr) {
+static void apc_bd_free(void *ptr TSRMLS_DC) {
     size_t *size;
-    TSRMLS_FETCH();
     if(zend_hash_index_find(&APCG(apc_bd_alloc_list), (ulong)ptr, (void**)&size) == FAILURE) {
-        apc_eprint("apc_bd_free could not free pointer (not found in list: %x)", ptr);
+        apc_error("apc_bd_free could not free pointer (not found in list: %x)" TSRMLS_CC, ptr);
+        return;
     }
     APCG(apc_bd_alloc_ptr) = (void*)((size_t)APCG(apc_bd_alloc_ptr) - *size);
     zend_hash_index_del(&APCG(apc_bd_alloc_list), (ulong)ptr);
@@ -133,9 +135,8 @@ static void apc_bd_free(void *ptr) {
  *  with a range of size.  If ptr_new is NULL, returns the next available
  *  block of given size.
  */
-static void *apc_bd_alloc_ex(void *ptr_new, size_t size) {
+static void *apc_bd_alloc_ex(void *ptr_new, size_t size TSRMLS_DC) {
     void *rval;
-    TSRMLS_FETCH();
 
     rval = APCG(apc_bd_alloc_ptr);
     if(ptr_new != NULL) {  /* reset ptrs */
@@ -144,10 +145,11 @@ static void *apc_bd_alloc_ex(void *ptr_new, size_t size) {
     } else {  /* alloc block */
       APCG(apc_bd_alloc_ptr) = (void*)((size_t)APCG(apc_bd_alloc_ptr) + size);
 #if APC_BINDUMP_DEBUG
-      apc_nprint("apc_bd_alloc: rval: 0x%x  ptr: 0x%x  ubptr: 0x%x  size: %d", rval, APCG(apc_bd_alloc_ptr), APCG(apc_bd_alloc_ubptr), size);
+      apc_notice("apc_bd_alloc: rval: 0x%x  ptr: 0x%x  ubptr: 0x%x  size: %d" TSRMLS_CC, rval, APCG(apc_bd_alloc_ptr), APCG(apc_bd_alloc_ubptr), size);
 #endif
       if(APCG(apc_bd_alloc_ptr) > APCG(apc_bd_alloc_ubptr)) {
-          apc_eprint("Exceeded bounds check in apc_bd_alloc_ex by %d bytes.", (unsigned char *) APCG(apc_bd_alloc_ptr) - (unsigned char *) APCG(apc_bd_alloc_ubptr));
+          apc_error("Exceeded bounds check in apc_bd_alloc_ex by %d bytes." TSRMLS_CC, (unsigned char *) APCG(apc_bd_alloc_ptr) - (unsigned char *) APCG(apc_bd_alloc_ubptr));
+          return NULL;
       }
       zend_hash_index_update(&APCG(apc_bd_alloc_list), (ulong)rval, &size, sizeof(size_t), NULL);
     }
@@ -166,7 +168,8 @@ static void _apc_swizzle_ptr(apc_bd_t *bd, zend_llist *ll, void **ptr, const cha
             printf(" in %s on line %d \n", file, line);
 #endif
         } else if((ulong)ptr > bd->size) {
-            apc_eprint("pointer to be swizzled is not within allowed memory range! (%x < %x < %x) in %s on %d", (long)bd, *ptr, ((long)bd + bd->size), file, line); \
+            apc_error("pointer to be swizzled is not within allowed memory range! (%x < %x < %x) in %s on %d" TSRMLS_CC, (long)bd, *ptr, ((long)bd + bd->size), file, line); \
+            return;
         }
     }
 } /* }}} */
@@ -191,6 +194,7 @@ static void apc_swizzle_op_array(apc_bd_t *bd, zend_llist *ll, zend_op_array *op
 
     /* swizzle op_array */
     for(i=0; i < op_array->last; i++) {
+#ifndef ZEND_ENGINE_2_4
         if(op_array->opcodes[i].result.op_type == IS_CONST) {
             apc_swizzle_zval(bd, ll, &op_array->opcodes[i].result.u.constant TSRMLS_CC);
         }
@@ -200,14 +204,23 @@ static void apc_swizzle_op_array(apc_bd_t *bd, zend_llist *ll, zend_op_array *op
         if(op_array->opcodes[i].op2.op_type == IS_CONST) {
             apc_swizzle_zval(bd, ll, &op_array->opcodes[i].op2.u.constant TSRMLS_CC);
         }
+#endif
         switch (op_array->opcodes[i].opcode) {
             case ZEND_JMP:
+#ifdef ZEND_ENGINE_2_4
+                apc_swizzle_ptr(bd, ll, &op_array->opcodes[i].op1.jmp_addr);
+#else
                 apc_swizzle_ptr(bd, ll, &op_array->opcodes[i].op1.u.jmp_addr);
+#endif
             case ZEND_JMPZ:
             case ZEND_JMPNZ:
             case ZEND_JMPZ_EX:
             case ZEND_JMPNZ_EX:
+#ifdef ZEND_ENGINE_2_4
+                apc_swizzle_ptr(bd, ll, &op_array->opcodes[i].op2.jmp_addr);
+#else
                 apc_swizzle_ptr(bd, ll, &op_array->opcodes[i].op2.u.jmp_addr);
+#endif
         }
     }
     apc_swizzle_ptr(bd, ll, &op_array->opcodes);
@@ -270,8 +283,8 @@ static void apc_swizzle_class_entry(apc_bd_t *bd, zend_llist *ll, zend_class_ent
         apc_swizzle_ptr(bd, ll, &ce->name);
     }
 
-    if(ce->doc_comment) {
-        apc_swizzle_ptr(bd, ll, &ce->doc_comment);
+    if (ce->type == ZEND_USER_CLASS && ZEND_CE_DOC_COMMENT(ce)) {
+        apc_swizzle_ptr(bd, ll, &ZEND_CE_DOC_COMMENT(ce));
     }
 
 #ifndef ZEND_ENGINE_2
@@ -279,12 +292,38 @@ static void apc_swizzle_class_entry(apc_bd_t *bd, zend_llist *ll, zend_class_ent
 #endif
 
     apc_swizzle_hashtable(bd, ll, &ce->function_table, (apc_swizzle_cb_t)apc_swizzle_function, 0 TSRMLS_CC);
+#ifdef ZEND_ENGINE_2_4
+    if (ce->default_properties_table) {
+        int i;
+
+        for (i = 0; i < ce->default_properties_count; i++) {
+            if (ce->default_properties_table[i]) {
+                apc_swizzle_ptr(bd, ll, &ce->default_properties_table[i]);
+                apc_swizzle_zval(bd, ll, ce->default_properties_table[i] TSRMLS_CC);
+            }
+        }
+    }
+#else
     apc_swizzle_hashtable(bd, ll, &ce->default_properties, (apc_swizzle_cb_t)apc_swizzle_zval, 1 TSRMLS_CC);
+#endif
 
 #ifdef ZEND_ENGINE_2
     apc_swizzle_hashtable(bd, ll, &ce->properties_info, (apc_swizzle_cb_t)apc_swizzle_property_info, 0 TSRMLS_CC);
 #endif
 
+#ifdef ZEND_ENGINE_2_4
+    if (ce->default_static_members_table) {
+        int i;
+
+        for (i = 0; i < ce->default_static_members_count; i++) {
+            if (ce->default_static_members_table[i]) {
+                apc_swizzle_ptr(bd, ll, &ce->default_static_members_table[i]);
+                apc_swizzle_zval(bd, ll, ce->default_static_members_table[i] TSRMLS_CC);
+            }
+        }
+    }
+    ce->static_members_table = ce->default_static_members_table;
+#else
     apc_swizzle_hashtable(bd, ll, &ce->default_static_members, (apc_swizzle_cb_t)apc_swizzle_zval, 1 TSRMLS_CC);
 
     if(ce->static_members != &ce->default_static_members) {
@@ -292,12 +331,13 @@ static void apc_swizzle_class_entry(apc_bd_t *bd, zend_llist *ll, zend_class_ent
     } else {
         apc_swizzle_ptr(bd, ll, &ce->static_members);
     }
+#endif
 
     apc_swizzle_hashtable(bd, ll, &ce->constants_table, (apc_swizzle_cb_t)apc_swizzle_zval, 1 TSRMLS_CC);
 
-    if(ce->builtin_functions) {
-        for(i=0; ce->builtin_functions[i].fname; i++) {
-            apc_swizzle_function_entry(bd, ll, &ce->builtin_functions[i] TSRMLS_CC);
+    if(ce->type == ZEND_INTERNAL_CLASS &&  ZEND_CE_BUILTIN_FUNCTIONS(ce)) {
+        for(i=0; ZEND_CE_BUILTIN_FUNCTIONS(ce)[i].fname; i++) {
+            apc_swizzle_function_entry(bd, ll, &ZEND_CE_BUILTIN_FUNCTIONS(ce)[i] TSRMLS_CC);
         }
     }
 
@@ -316,7 +356,9 @@ static void apc_swizzle_class_entry(apc_bd_t *bd, zend_llist *ll, zend_class_ent
     apc_swizzle_ptr(bd, ll, &ce->__tostring);
 #endif
 
-    apc_swizzle_ptr(bd, ll, &ce->filename);
+    if (ce->type == ZEND_USER_CLASS) {
+        apc_swizzle_ptr(bd, ll, &ZEND_CE_FILENAME(ce));
+    }
 } /* }}} */
 
 
@@ -513,8 +555,7 @@ static int apc_unswizzle_bd(apc_bd_t *bd, int flags TSRMLS_DC) {
         PHP_MD5Update(&context, (const unsigned char*)bd, bd->size);
         PHP_MD5Final(digest, &context);
         if(memcmp(md5_orig, digest, 16)) {
-            apc_eprint("MD5 checksum of binary dump failed.");
-            memcpy(bd->md5, md5_orig, 16); /* add back md5 checksum */
+            apc_error("MD5 checksum of binary dump failed." TSRMLS_CC);
             return -1;
         }
     }
@@ -525,8 +566,7 @@ static int apc_unswizzle_bd(apc_bd_t *bd, int flags TSRMLS_DC) {
           crc = ((crc >> 8) & 0x00FFFFFF) ^ crc32tab[(crc ^ (*crc_p)) & 0xFF ];
         }
         if(crc_orig != crc) {
-            apc_eprint("CRC32 checksum of binary dump failed.");
-            bd->crc = crc_orig;
+            apc_error("CRC32 checksum of binary dump failed." TSRMLS_CC);
             return -1;
         }
     }
@@ -593,12 +633,16 @@ static inline void apc_bin_fixup_class_entry(zend_class_entry *ce) {
 
     /* fixup hashtable destructor pointers */
     ce->function_table.pDestructor = (dtor_func_t)zend_function_dtor;
+#ifndef ZEND_ENGINE_2_4
     ce->default_properties.pDestructor = (dtor_func_t)zval_ptr_dtor_wrapper;
+#endif
     ce->properties_info.pDestructor = (dtor_func_t)zval_ptr_dtor_wrapper;
+#ifndef ZEND_ENGINE_2_4
     ce->default_static_members.pDestructor = (dtor_func_t)zval_ptr_dtor_wrapper;
     if (ce->static_members) {
         ce->static_members->pDestructor = (dtor_func_t)zval_ptr_dtor_wrapper;
     }
+#endif
     ce->constants_table.pDestructor = (dtor_func_t)zval_ptr_dtor_wrapper;
 }
 /* }}} */
@@ -629,7 +673,7 @@ apc_bd_t* apc_bin_dump(HashTable *files, HashTable *user_vars TSRMLS_DC) {
     for(i=0; i < apc_user_cache->num_slots; i++) {
         sp = apc_user_cache->slots[i];
         for(; sp != NULL; sp = sp->next) {
-            if(apc_bin_checkfilter(user_vars, sp->key.data.user.identifier, sp->key.data.user.identifier_len+1)) {
+            if(apc_bin_checkfilter(user_vars, sp->key.data.user.identifier, sp->key.data.user.identifier_len)) {
                 size += sizeof(apc_bd_entry_t*) + sizeof(apc_bd_entry_t);
                 size += sp->value->mem_size - (sizeof(apc_cache_entry_t) - sizeof(apc_cache_entry_value_t));
                 count++;
@@ -647,7 +691,7 @@ apc_bd_t* apc_bin_dump(HashTable *files, HashTable *user_vars TSRMLS_DC) {
                 }
             } else {
                 /* TODO: Currently we don't support APC_CACHE_KEY_FILE type.  We need to store the path and re-stat on load */
-                apc_wprint("Excluding some files from apc_bin_dump[file].  Cached files must be included using full path with apc.stat=0.");
+                apc_warning("Excluding some files from apc_bin_dump[file].  Cached files must be included using full path with apc.stat=0." TSRMLS_CC);
             }
         }
     }
@@ -656,16 +700,16 @@ apc_bd_t* apc_bin_dump(HashTable *files, HashTable *user_vars TSRMLS_DC) {
     bd = emalloc(size);
     bd->size = size;
     pool_ptr = emalloc(sizeof(apc_pool));
-    apc_bd_alloc_ex(pool_ptr, sizeof(apc_pool));
-    ctxt.pool = apc_pool_create(APC_UNPOOL, apc_bd_alloc, apc_bd_free, NULL, NULL);  /* ideally the pool wouldn't be alloc'd as part of this */
+    apc_bd_alloc_ex(pool_ptr, sizeof(apc_pool) TSRMLS_CC);
+    ctxt.pool = apc_pool_create(APC_UNPOOL, apc_bd_alloc, apc_bd_free, NULL, NULL TSRMLS_CC);  /* ideally the pool wouldn't be alloc'd as part of this */
     if (!ctxt.pool) { /* TODO need to cleanup */
-        apc_wprint("Unable to allocate memory for pool.");
+        apc_warning("Unable to allocate memory for pool." TSRMLS_CC);
         return NULL;
     }
     ctxt.copy = APC_COPY_IN_USER;  /* avoid stupid ALLOC_ZVAL calls here, hack */
-    apc_bd_alloc_ex( (void*)((long)bd + sizeof(apc_bd_t)), bd->size - sizeof(apc_bd_t) -1);
+    apc_bd_alloc_ex((void*)((long)bd + sizeof(apc_bd_t)), bd->size - sizeof(apc_bd_t) -1 TSRMLS_CC);
     bd->num_entries = count;
-    bd->entries = apc_bd_alloc_ex(NULL, sizeof(apc_bd_entry_t) * count);
+    bd->entries = apc_bd_alloc_ex(NULL, sizeof(apc_bd_entry_t) * count TSRMLS_CC);
 
     /* User entries */
     zend_hash_init(&APCG(copied_zvals), 0, NULL, NULL, 0);
@@ -673,13 +717,13 @@ apc_bd_t* apc_bin_dump(HashTable *files, HashTable *user_vars TSRMLS_DC) {
     for(i=0; i < apc_user_cache->num_slots; i++) {
         sp = apc_user_cache->slots[i];
         for(; sp != NULL; sp = sp->next) {
-            if(apc_bin_checkfilter(user_vars, sp->key.data.user.identifier, sp->key.data.user.identifier_len+1)) {
+            if(apc_bin_checkfilter(user_vars, sp->key.data.user.identifier, sp->key.data.user.identifier_len)) {
                 ep = &bd->entries[count];
                 ep->type = sp->value->type;
-                ep->val.user.info = apc_bd_alloc(sp->value->data.user.info_len+1);
-                memcpy(ep->val.user.info, sp->value->data.user.info, sp->value->data.user.info_len+1);
+                ep->val.user.info = apc_bd_alloc(sp->value->data.user.info_len TSRMLS_CC);
+                memcpy(ep->val.user.info, sp->value->data.user.info, sp->value->data.user.info_len);
                 ep->val.user.info_len = sp->value->data.user.info_len;
-                ep->val.user.val = apc_copy_zval(NULL, sp->value->data.user.val, &ctxt);
+                ep->val.user.val = apc_copy_zval(NULL, sp->value->data.user.val, &ctxt TSRMLS_CC);
                 ep->val.user.ttl = sp->value->data.user.ttl;
 
                 /* swizzle pointers */
@@ -702,17 +746,17 @@ apc_bd_t* apc_bin_dump(HashTable *files, HashTable *user_vars TSRMLS_DC) {
                 if(apc_bin_checkfilter(files, sp->key.data.fpfile.fullpath, sp->key.data.fpfile.fullpath_len+1)) {
                     ep = &bd->entries[count];
                     ep->type = sp->key.type;
-                    ep->val.file.filename = apc_bd_alloc(strlen(sp->value->data.file.filename)+1);
+                    ep->val.file.filename = apc_bd_alloc(strlen(sp->value->data.file.filename) + 1 TSRMLS_CC);
                     strcpy(ep->val.file.filename, sp->value->data.file.filename);
                     ep->val.file.op_array = apc_copy_op_array(NULL, sp->value->data.file.op_array, &ctxt TSRMLS_CC);
 
                     for(ep->num_functions=0; sp->value->data.file.functions[ep->num_functions].function != NULL;) { ep->num_functions++; }
-                    ep->val.file.functions = apc_bd_alloc(sizeof(apc_function_t) * ep->num_functions);
+                    ep->val.file.functions = apc_bd_alloc(sizeof(apc_function_t) * ep->num_functions TSRMLS_CC);
                     for(fcount=0; fcount < ep->num_functions; fcount++) {
                         memcpy(&ep->val.file.functions[fcount], &sp->value->data.file.functions[fcount], sizeof(apc_function_t));
-                        ep->val.file.functions[fcount].name = apc_xmemcpy(sp->value->data.file.functions[fcount].name, sp->value->data.file.functions[fcount].name_len+1, apc_bd_alloc);
+                        ep->val.file.functions[fcount].name = apc_xmemcpy(sp->value->data.file.functions[fcount].name, sp->value->data.file.functions[fcount].name_len+1, apc_bd_alloc TSRMLS_CC);
                         ep->val.file.functions[fcount].name_len = sp->value->data.file.functions[fcount].name_len;
-                        ep->val.file.functions[fcount].function = apc_bd_alloc(sizeof(zend_function));
+                        ep->val.file.functions[fcount].function = apc_bd_alloc(sizeof(zend_function) TSRMLS_CC);
                         efp = ep->val.file.functions[fcount].function;
                         sfp = sp->value->data.file.functions[fcount].function;
                         switch(sfp->type) {
@@ -738,12 +782,12 @@ apc_bd_t* apc_bin_dump(HashTable *files, HashTable *user_vars TSRMLS_DC) {
 
 
                     for(ep->num_classes=0; sp->value->data.file.classes[ep->num_classes].class_entry != NULL;) { ep->num_classes++; }
-                    ep->val.file.classes = apc_bd_alloc(sizeof(apc_class_t) * ep->num_classes);
+                    ep->val.file.classes = apc_bd_alloc(sizeof(apc_class_t) * ep->num_classes TSRMLS_CC);
                     for(fcount=0; fcount < ep->num_classes; fcount++) {
-                        ep->val.file.classes[fcount].name = apc_xmemcpy(sp->value->data.file.classes[fcount].name, sp->value->data.file.classes[fcount].name_len+1, apc_bd_alloc);
+                        ep->val.file.classes[fcount].name = apc_xmemcpy(sp->value->data.file.classes[fcount].name, sp->value->data.file.classes[fcount].name_len + 1, apc_bd_alloc TSRMLS_CC);
                         ep->val.file.classes[fcount].name_len = sp->value->data.file.classes[fcount].name_len;
-                        ep->val.file.classes[fcount].class_entry = apc_copy_class_entry(NULL, sp->value->data.file.classes[fcount].class_entry, &ctxt);
-                        ep->val.file.classes[fcount].parent_name = apc_xstrdup(sp->value->data.file.classes[fcount].parent_name, apc_bd_alloc);
+                        ep->val.file.classes[fcount].class_entry = apc_copy_class_entry(NULL, sp->value->data.file.classes[fcount].class_entry, &ctxt TSRMLS_CC);
+                        ep->val.file.classes[fcount].parent_name = apc_xstrdup(sp->value->data.file.classes[fcount].parent_name, apc_bd_alloc TSRMLS_CC);
 
                         apc_swizzle_ptr(bd, &ll, &ep->val.file.classes[fcount].name);
                         apc_swizzle_ptr(bd, &ll, &ep->val.file.classes[fcount].parent_name);
@@ -808,9 +852,9 @@ int apc_bin_load(apc_bd_t *bd, int flags TSRMLS_DC) {
     t = apc_time();
 
     for(i = 0; i < bd->num_entries; i++) {
-        ctxt.pool = apc_pool_create(APC_SMALL_POOL, apc_sma_malloc, apc_sma_free, apc_sma_protect, apc_sma_unprotect);
+        ctxt.pool = apc_pool_create(APC_SMALL_POOL, apc_sma_malloc, apc_sma_free, apc_sma_protect, apc_sma_unprotect TSRMLS_CC);
         if (!ctxt.pool) { /* TODO need to cleanup previous pools */
-            apc_wprint("Unable to allocate memory for pool.");
+            apc_warning("Unable to allocate memory for pool." TSRMLS_CC);
             goto failure;
         }
         ep = &bd->entries[i];
@@ -824,7 +868,7 @@ int apc_bin_load(apc_bd_t *bd, int flags TSRMLS_DC) {
                 HANDLE_BLOCK_INTERRUPTIONS();
 #if NONBLOCKING_LOCK_AVAILABLE
                 if(APCG(write_lock)) {
-                    if(!apc_cache_write_lock(apc_cache)) {
+                    if(!apc_cache_write_lock(apc_cache TSRMLS_CC)) {
                         HANDLE_UNBLOCK_INTERRUPTIONS();
                         return -1;
                     }
@@ -835,15 +879,15 @@ int apc_bin_load(apc_bd_t *bd, int flags TSRMLS_DC) {
                 }
                 apc_bin_fixup_op_array(alloc_op_array);
 
-                if(! (alloc_functions = apc_sma_malloc(sizeof(apc_function_t) * (ep->num_functions + 1)))) {
+                if(! (alloc_functions = apc_sma_malloc(sizeof(apc_function_t) * (ep->num_functions + 1) TSRMLS_CC))) {
                     goto failure;
                 }
                 for(i2=0; i2 < ep->num_functions; i2++) {
-                    if(! (alloc_functions[i2].name = apc_xmemcpy(ep->val.file.functions[i2].name, ep->val.file.functions[i2].name_len+1, apc_sma_malloc))) {
+                    if(! (alloc_functions[i2].name = apc_xmemcpy(ep->val.file.functions[i2].name, ep->val.file.functions[i2].name_len + 1, apc_sma_malloc TSRMLS_CC))) {
                         goto failure;
                     }
                     alloc_functions[i2].name_len = ep->val.file.functions[i2].name_len;
-                    if(! (alloc_functions[i2].function = apc_sma_malloc(sizeof(zend_function)))) {
+                    if(! (alloc_functions[i2].function = apc_sma_malloc(sizeof(zend_function) TSRMLS_CC))) {
                         goto failure;
                     }
                     switch(ep->val.file.functions[i2].function->type) {
@@ -869,19 +913,19 @@ int apc_bin_load(apc_bd_t *bd, int flags TSRMLS_DC) {
                 alloc_functions[i2].name = NULL;
                 alloc_functions[i2].function = NULL;
 
-                if(! (alloc_classes = apc_sma_malloc(sizeof(apc_class_t) * (ep->num_classes + 1)))) {
+                if(! (alloc_classes = apc_sma_malloc(sizeof(apc_class_t) * (ep->num_classes + 1) TSRMLS_CC))) {
                     goto failure;
                 }
                 for(i2=0; i2 < ep->num_classes; i2++) {
-                    if(! (alloc_classes[i2].name = apc_xmemcpy(ep->val.file.classes[i2].name, ep->val.file.classes[i2].name_len+1, apc_sma_malloc))) {
+                    if(! (alloc_classes[i2].name = apc_xmemcpy(ep->val.file.classes[i2].name, ep->val.file.classes[i2].name_len+1, apc_sma_malloc TSRMLS_CC))) {
                         goto failure;
                     }
                     alloc_classes[i2].name_len = ep->val.file.classes[i2].name_len;
-                    if(! (alloc_classes[i2].class_entry = apc_copy_class_entry(NULL, ep->val.file.classes[i2].class_entry, &ctxt))) {
+                    if(! (alloc_classes[i2].class_entry = apc_copy_class_entry(NULL, ep->val.file.classes[i2].class_entry, &ctxt TSRMLS_CC))) {
                         goto failure;
                     }
                     apc_bin_fixup_class_entry(alloc_classes[i2].class_entry);
-                    if(! (alloc_classes[i2].parent_name = apc_xstrdup(ep->val.file.classes[i2].parent_name, apc_sma_malloc))) {
+                    if(! (alloc_classes[i2].parent_name = apc_xstrdup(ep->val.file.classes[i2].parent_name, apc_sma_malloc TSRMLS_CC))) {
                         if(ep->val.file.classes[i2].parent_name != NULL) {
                             goto failure;
                         }
@@ -890,7 +934,7 @@ int apc_bin_load(apc_bd_t *bd, int flags TSRMLS_DC) {
                 alloc_classes[i2].name = NULL;
                 alloc_classes[i2].class_entry = NULL;
 
-                if(!(cache_entry = apc_cache_make_file_entry(ep->val.file.filename, alloc_op_array, alloc_functions, alloc_classes, &ctxt))) {
+                if(!(cache_entry = apc_cache_make_file_entry(ep->val.file.filename, alloc_op_array, alloc_functions, alloc_classes, &ctxt TSRMLS_CC))) {
                     goto failure;
                 }
 
@@ -898,7 +942,7 @@ int apc_bin_load(apc_bd_t *bd, int flags TSRMLS_DC) {
                     goto failure;
                 }
 
-                if ((ret = apc_cache_insert(apc_cache, cache_key, cache_entry, &ctxt, t)) != 1) {
+                if ((ret = apc_cache_insert(apc_cache, cache_key, cache_entry, &ctxt, t TSRMLS_CC)) != 1) {
                     if(ret==-1) {
                         goto failure;
                     }
@@ -906,7 +950,7 @@ int apc_bin_load(apc_bd_t *bd, int flags TSRMLS_DC) {
 
 #if NONBLOCKING_LOCK_AVAILABLE
                 if(APCG(write_lock)) {
-                    apc_cache_write_unlock(apc_cache);
+                    apc_cache_write_unlock(apc_cache TSRMLS_CC);
                 }
 #endif
                 HANDLE_UNBLOCK_INTERRUPTIONS();
@@ -924,11 +968,11 @@ int apc_bin_load(apc_bd_t *bd, int flags TSRMLS_DC) {
     return 0;
 
 failure:
-    apc_pool_destroy(ctxt.pool);
-    apc_wprint("Unable to allocate memory for apc binary load/dump functionality.");
+    apc_pool_destroy(ctxt.pool TSRMLS_CC);
+    apc_warning("Unable to allocate memory for apc binary load/dump functionality." TSRMLS_CC);
 #if NONBLOCKING_LOCK_AVAILABLE
     if(APCG(write_lock)) {
-        apc_cache_write_unlock(apc_cache);
+        apc_cache_write_unlock(apc_cache TSRMLS_CC);
     }
 #endif
     HANDLE_UNBLOCK_INTERRUPTIONS();
