@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2008 The PHP Group                                     |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -26,7 +26,7 @@
 
  */
 
-/* $Id: apc_cache.h,v 3.46.2.4 2008/05/11 18:57:00 rasmus Exp $ */
+/* $Id: apc_cache.h 307048 2011-01-03 23:53:17Z kalle $ */
 
 #ifndef APC_CACHE_H
 #define APC_CACHE_H
@@ -39,6 +39,9 @@
 #include "apc.h"
 #include "apc_compile.h"
 #include "apc_lock.h"
+#include "apc_pool.h"
+#include "apc_main.h"
+#include "TSRM.h"
 
 #define APC_CACHE_ENTRY_FILE   1
 #define APC_CACHE_ENTRY_USER   2
@@ -47,14 +50,44 @@
 #define APC_CACHE_KEY_USER     2
 #define APC_CACHE_KEY_FPFILE   3
 
+#ifdef PHP_WIN32
+typedef unsigned __int64 apc_ino_t;
+typedef unsigned __int64 apc_dev_t;
+#else
+typedef ino_t apc_ino_t;
+typedef dev_t apc_dev_t;
+#endif
+
+/* {{{ cache locking macros */
+#define CACHE_LOCK(cache)        { LOCK(cache->header->lock);   cache->has_lock = 1; }
+#define CACHE_UNLOCK(cache)      { UNLOCK(cache->header->lock); cache->has_lock = 0; }
+#define CACHE_SAFE_LOCK(cache)   { if ((++cache->has_lock) == 1) LOCK(cache->header->lock); }
+#define CACHE_SAFE_UNLOCK(cache) { if ((--cache->has_lock) == 0) UNLOCK(cache->header->lock); }
+
+#if (RDLOCK_AVAILABLE == 1) && defined(HAVE_ATOMIC_OPERATIONS)
+#define USE_READ_LOCKS 1
+#define CACHE_RDLOCK(cache)      { RDLOCK(cache->header->lock);  cache->has_lock = 0; }
+#define CACHE_SAFE_INC(cache, obj) { ATOMIC_INC(obj); }
+#define CACHE_SAFE_DEC(cache, obj) { ATOMIC_DEC(obj); }
+#else
+#define USE_READ_LOCKS 0
+#define CACHE_RDLOCK(cache)      { LOCK(cache->header->lock);  cache->has_lock = 1; }
+#define CACHE_SAFE_INC(cache, obj) { CACHE_SAFE_LOCK(cache); obj++; CACHE_SAFE_UNLOCK(cache);}
+#define CACHE_SAFE_DEC(cache, obj) { CACHE_SAFE_LOCK(cache); obj--; CACHE_SAFE_UNLOCK(cache);}
+#endif
+
+#define CACHE_FAST_INC(cache, obj) { obj++; }
+#define CACHE_FAST_DEC(cache, obj) { obj--; }
+/* }}} */
+
 /* {{{ struct definition: apc_cache_key_t */
 #define T apc_cache_t*
 typedef struct apc_cache_t apc_cache_t; /* opaque cache type */
 
 typedef union _apc_cache_key_data_t {
     struct {
-        dev_t device;             /* the filesystem device */
-        ino_t inode;              /* the filesystem inode */
+        apc_dev_t device;             /* the filesystem device */
+        apc_ino_t inode;              /* the filesystem inode */
     } file;
     struct {
         const char *identifier;
@@ -71,6 +104,21 @@ struct apc_cache_key_t {
     apc_cache_key_data_t data;
     time_t mtime;                 /* the mtime of this cached entry */
     unsigned char type;
+    unsigned char md5[16];        /* md5 hash of the source file */
+};
+
+
+typedef struct apc_keyid_t apc_keyid_t;
+
+struct apc_keyid_t {
+    unsigned int h;
+    unsigned int keylen;
+    time_t mtime;
+#ifdef ZTS
+    THREAD_T tid;
+#else
+    pid_t pid;
+#endif
 };
 /* }}} */
 
@@ -81,10 +129,11 @@ typedef union _apc_cache_entry_value_t {
         zend_op_array* op_array;    /* op_array allocated in shared memory */
         apc_function_t* functions;  /* array of apc_function_t's */
         apc_class_t* classes;       /* array of apc_class_t's */
+        long halt_offset;           /* value of __COMPILER_HALT_OFFSET__ for the file */
     } file;
     struct {
-        char *info; 
-        int info_len; 
+        char *info;
+        int info_len;
         zval *val;
         unsigned int ttl;
     } user;
@@ -96,6 +145,7 @@ struct apc_cache_entry_t {
     unsigned char type;
     int ref_count;
     size_t mem_size;
+    apc_pool *pool;
 };
 /* }}} */
 
@@ -117,20 +167,20 @@ struct apc_cache_entry_t {
  * is needed.  This helps in cleaning up the cache and ensuring that entries 
  * hit frequently stay cached and ones not hit very often eventually disappear.
  */
-extern T apc_cache_create(int size_hint, int gc_ttl, int ttl);
+extern T apc_cache_create(int size_hint, int gc_ttl, int ttl TSRMLS_DC);
 
 /*
  * apc_cache_destroy releases any OS resources associated with a cache object.
  * Under apache, this function can be safely called by the child processes
  * when they exit.
  */
-extern void apc_cache_destroy(T cache);
+extern void apc_cache_destroy(T cache TSRMLS_DC);
 
 /*
  * apc_cache_clear empties a cache. This can safely be called at any time,
  * even while other server processes are executing cached source files.
  */
-extern void apc_cache_clear(T cache);
+extern void apc_cache_clear(T cache TSRMLS_DC);
 
 /*
  * apc_cache_insert adds an entry to the cache, using a filename as a key.
@@ -145,10 +195,13 @@ extern void apc_cache_clear(T cache);
  * value is a cache entry returned by apc_cache_make_entry (see below).
  */
 extern int apc_cache_insert(T cache, apc_cache_key_t key,
-                            apc_cache_entry_t* value, time_t t);
+                            apc_cache_entry_t* value, apc_context_t* ctxt, time_t t TSRMLS_DC);
 
 extern int apc_cache_user_insert(T cache, apc_cache_key_t key,
-                            apc_cache_entry_t* value, time_t t, int exclusive TSRMLS_DC);
+                            apc_cache_entry_t* value, apc_context_t* ctxt, time_t t, int exclusive TSRMLS_DC);
+
+extern int *apc_cache_insert_mult(apc_cache_t* cache, apc_cache_key_t* keys,
+                            apc_cache_entry_t** values, apc_context_t *ctxt, time_t t, int num_entries TSRMLS_DC);
 
 /*
  * apc_cache_find searches for a cache entry by filename, and returns a
@@ -156,25 +209,35 @@ extern int apc_cache_user_insert(T cache, apc_cache_key_t key,
  *
  * key is a value created by apc_cache_make_file_key for file keys.
  */
-extern apc_cache_entry_t* apc_cache_find(T cache, apc_cache_key_t key, time_t t);
+extern apc_cache_entry_t* apc_cache_find(T cache, apc_cache_key_t key, time_t t TSRMLS_DC);
 
 /*
- * apc_cache_user_find searches for a cache entry by its hashed identifier, 
+ * apc_cache_user_find searches for a cache entry by its hashed identifier,
  * and returns a pointer to the entry if found, NULL otherwise.
  *
  */
-extern apc_cache_entry_t* apc_cache_user_find(T cache, char* strkey, int keylen, time_t t);
+extern apc_cache_entry_t* apc_cache_user_find(T cache, char* strkey, int keylen, time_t t TSRMLS_DC);
 
 /*
- * apc_cache_user_delete finds an entry in the user cache and deletes it.
+ * apc_cache_user_exists searches for a cache entry by its hashed identifier,
+ * and returns a pointer to the entry if found, NULL otherwise.  This is a
+ * quick non-locking version of apc_cache_user_find that does not modify the
+ * shared memory segment in any way.
+ *
  */
-extern int apc_cache_user_delete(apc_cache_t* cache, char *strkey, int keylen);
+extern apc_cache_entry_t* apc_cache_user_exists(T cache, char* strkey, int keylen, time_t t TSRMLS_DC);
+
+/*
+ * apc_cache_delete and apc_cache_user_delete finds an entry in the cache and deletes it.
+ */
+extern int apc_cache_delete(apc_cache_t* cache, char *filename, int filename_len TSRMLS_DC);
+extern int apc_cache_user_delete(apc_cache_t* cache, char *strkey, int keylen TSRMLS_DC);
 
 /* apc_cach_fetch_zval takes a zval in the cache and reconstructs a runtime
  * zval from it.
  *
  */
-zval* apc_cache_fetch_zval(zval* dst, const zval* src, apc_malloc_t allocate, apc_free_t deallocate);
+zval* apc_cache_fetch_zval(zval* dst, const zval* src, apc_context_t* ctxt TSRMLS_DC);
 
 /*
  * apc_cache_release decrements the reference count associated with a cache
@@ -185,7 +248,7 @@ zval* apc_cache_fetch_zval(zval* dst, const zval* src, apc_malloc_t allocate, ap
  *
  * entry is the cache entry whose ref count you want to decrement.
  */
-extern void apc_cache_release(T cache, apc_cache_entry_t* entry);
+extern void apc_cache_release(T cache, apc_cache_entry_t* entry TSRMLS_DC);
 
 /*
  * apc_cache_make_file_key creates a key object given a relative or absolute
@@ -207,7 +270,7 @@ extern int apc_cache_make_file_key(apc_cache_key_t* key,
                                    const char* filename,
                                    const char* include_path,
                                    time_t t
-							       TSRMLS_DC);
+                                   TSRMLS_DC);
 
 /*
  * apc_cache_make_file_entry creates an apc_cache_entry_t object given a filename
@@ -216,75 +279,28 @@ extern int apc_cache_make_file_key(apc_cache_key_t* key,
 extern apc_cache_entry_t* apc_cache_make_file_entry(const char* filename,
                                                     zend_op_array* op_array,
                                                     apc_function_t* functions,
-                                                    apc_class_t* classes);
+                                                    apc_class_t* classes,
+                                                    apc_context_t* ctxt
+                                                    TSRMLS_DC);
+
+
+zend_bool apc_compile_cache_entry(apc_cache_key_t key, zend_file_handle* h, int type, time_t t, zend_op_array** op_array_pp, apc_cache_entry_t** cache_entry_pp TSRMLS_DC);
+
 /*
  * apc_cache_make_user_entry creates an apc_cache_entry_t object given an info string
  * and the zval to be stored.
  */
-extern apc_cache_entry_t* apc_cache_make_user_entry(const char* info, int info_len, const zval *val, const unsigned int ttl);
+extern apc_cache_entry_t* apc_cache_make_user_entry(const char* info, int info_len, const zval *val, apc_context_t* ctxt, const unsigned int ttl TSRMLS_DC);
 
 extern int apc_cache_make_user_key(apc_cache_key_t* key, char* identifier, int identifier_len, const time_t t);
 
-/*
- * Frees all memory associated with an object returned by apc_cache_make_entry
- * (see above).
- */
-extern void apc_cache_free_entry(apc_cache_entry_t* entry);
-
-/* {{{ struct definition: apc_cache_link_data_t */
-typedef union _apc_cache_link_data_t {
-    struct {
-        char *filename;
-        dev_t device;
-        ino_t inode;
-    } file;
-    struct {
-        char *info;
-        unsigned int ttl;
-    } user;
-} apc_cache_link_data_t;
-/* }}} */
-
-/* {{{ struct definition: apc_cache_link_t */
-typedef struct apc_cache_link_t apc_cache_link_t;
-struct apc_cache_link_t {
-    apc_cache_link_data_t data;
-    unsigned char type;
-    int num_hits;
-    time_t mtime;
-    time_t creation_time;
-    time_t deletion_time;
-    time_t access_time;
-    int ref_count;
-    size_t mem_size;
-    apc_cache_link_t* next;
-};
-/* }}} */
-
-/* {{{ struct definition: apc_cache_info_t */
-typedef struct apc_cache_info_t apc_cache_info_t;
-struct apc_cache_info_t {
-    int num_slots;
-    int num_hits;
-    int num_misses;
-    int ttl;
-    apc_cache_link_t* list;
-    apc_cache_link_t* deleted_list;
-    time_t start_time;
-    int expunges;
-    int num_entries;
-    int num_inserts;
-    size_t mem_size;
-};
-/* }}} */
-
 /* {{{ struct definition: slot_t */
 typedef struct slot_t slot_t;
 struct slot_t {
     apc_cache_key_t key;        /* slot key */
     apc_cache_entry_t* value;   /* slot value */
     slot_t* next;               /* next slot in linked list */
-    int num_hits;               /* number of hits to this bucket */
+    unsigned long num_hits;     /* number of hits to this bucket */
     time_t creation_time;       /* time slot was initialized */
     time_t deletion_time;       /* time slot was removed from cache */
     time_t access_time;         /* time slot was last accessed */
@@ -295,38 +311,49 @@ struct slot_t {
    Any values that must be shared among processes should go in here. */
 typedef struct cache_header_t cache_header_t;
 struct cache_header_t {
-    apc_lck_t lock;              /* read/write lock (exclusive blocking cache lock) */
+    apc_lck_t lock;             /* read/write lock (exclusive blocking cache lock) */
     apc_lck_t wrlock;           /* write lock (non-blocking used to prevent cache slams) */
-    int num_hits;               /* total successful hits in cache */
-    int num_misses;             /* total unsuccessful hits in cache */
-    int num_inserts;            /* total successful inserts in cache */
+    unsigned long num_hits;     /* total successful hits in cache */
+    unsigned long num_misses;   /* total unsuccessful hits in cache */
+    unsigned long num_inserts;  /* total successful inserts in cache */
+    unsigned long expunges;     /* total number of expunges */
     slot_t* deleted_list;       /* linked list of to-be-deleted slots */
     time_t start_time;          /* time the above counters were reset */
-    int expunges;               /* total number of expunges */
     zend_bool busy;             /* Flag to tell clients when we are busy cleaning the cache */
     int num_entries;            /* Statistic on the number of entries */
     size_t mem_size;            /* Statistic on the memory size used by this cache */
+    apc_keyid_t lastkey;        /* the key that is being inserted (user cache) */
 };
 /* }}} */
 
+typedef void (*apc_expunge_cb_t)(T cache, size_t n TSRMLS_DC); 
+
 /* {{{ struct definition: apc_cache_t */
 struct apc_cache_t {
-    void* shmaddr;              /* process (local) address of shared cache */
-    cache_header_t* header;           /* cache header (stored in SHM) */
-    slot_t** slots;             /* array of cache slots (stored in SHM) */
-    int num_slots;              /* number of slots in cache */
-    int gc_ttl;                 /* maximum time on GC list for a slot */
-    int ttl;                    /* if slot is needed and entry's access time is older than this ttl, remove it */
+    void* shmaddr;                /* process (local) address of shared cache */
+    cache_header_t* header;       /* cache header (stored in SHM) */
+    slot_t** slots;               /* array of cache slots (stored in SHM) */
+    int num_slots;                /* number of slots in cache */
+    int gc_ttl;                   /* maximum time on GC list for a slot */
+    int ttl;                      /* if slot is needed and entry's access time is older than this ttl, remove it */
+    apc_expunge_cb_t expunge_cb;  /* cache specific expunge callback to free up sma memory */
+    uint has_lock;                /* flag for possible recursive locks within the same process */
 };
 /* }}} */
 
-extern apc_cache_info_t* apc_cache_info(T cache, zend_bool limited);
-extern void apc_cache_free_info(apc_cache_info_t* info);
-extern void apc_cache_expunge(apc_cache_t* cache, time_t t);
-extern void apc_cache_unlock(apc_cache_t* cache);
+extern zval* apc_cache_info(T cache, zend_bool limited TSRMLS_DC);
+extern void apc_cache_unlock(apc_cache_t* cache TSRMLS_DC);
 extern zend_bool apc_cache_busy(apc_cache_t* cache);
-extern zend_bool apc_cache_write_lock(apc_cache_t* cache);
-extern void apc_cache_write_unlock(apc_cache_t* cache);
+extern zend_bool apc_cache_write_lock(apc_cache_t* cache TSRMLS_DC);
+extern void apc_cache_write_unlock(apc_cache_t* cache TSRMLS_DC);
+extern zend_bool apc_cache_is_last_key(apc_cache_t* cache, apc_cache_key_t* key, unsigned int h, time_t t TSRMLS_DC);
+
+/* used by apc_rfc1867 to update data in-place - not to be used elsewhere */
+
+typedef int (*apc_cache_updater_t)(apc_cache_t*, apc_cache_entry_t*, void* data);
+extern int _apc_cache_user_update(apc_cache_t* cache, char *strkey, int keylen,
+                                    apc_cache_updater_t updater, void* data TSRMLS_DC);
+
 
 #undef T
 #endif
@@ -336,6 +363,6 @@ extern void apc_cache_write_unlock(apc_cache_t* cache);
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: expandtab sw=4 ts=4 sts=4 fdm=marker
+ * vim>600: expandtab sw=4 ts=4 sts=4 fdm=marker
  * vim<600: expandtab sw=4 ts=4 sts=4
  */
