@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2008 The PHP Group                                     |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -12,12 +12,12 @@
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
-  | Authors: Brian Shire <shire@.php.net>                                |
+  | Authors: Brian Shire <shire@php.net>                                 |
   +----------------------------------------------------------------------+
 
  */
 
-/* $Id: apc_iterator.c 284698 2009-07-24 11:35:52Z kalle $ */
+/* $Id: apc_iterator.c 307265 2011-01-08 13:30:25Z gopalv $ */
 
 #include "php_apc.h"
 #include "apc_iterator.h"
@@ -33,7 +33,7 @@ zend_object_handlers apc_iterator_object_handlers;
 
 
 /* {{{ apc_iterator_item */
-static apc_iterator_item_t* apc_iterator_item_ctor(apc_iterator_t *iterator, slot_t **slot_pp) {
+static apc_iterator_item_t* apc_iterator_item_ctor(apc_iterator_t *iterator, slot_t **slot_pp TSRMLS_DC) {
     zval *zvalue;
     char md5str[33];
     slot_t *slot = *slot_pp;
@@ -42,7 +42,11 @@ static apc_iterator_item_t* apc_iterator_item_ctor(apc_iterator_t *iterator, slo
 
     if (slot->key.type == APC_CACHE_KEY_FILE) {
         /* keys should be unique and with stat=1 we could have multiple files with the same name, so use '<device> <inode>' instead */
+#ifdef PHP_WIN32
+        item->key_len = spprintf(&item->key, 0, "%I64d %I64d", slot->key.data.file.device, slot->key.data.file.inode);
+#else
         item->key_len = spprintf(&item->key, 0, "%ld %ld", (ulong)slot->key.data.file.device, (ulong)slot->key.data.file.inode);
+#endif
         item->filename_key = estrdup(slot->value->data.file.filename);
     } else if (slot->key.type == APC_CACHE_KEY_USER) {
         item->key = estrndup((char*)slot->key.data.user.identifier, slot->key.data.user.identifier_len);
@@ -51,8 +55,9 @@ static apc_iterator_item_t* apc_iterator_item_ctor(apc_iterator_t *iterator, slo
     } else if (slot->key.type == APC_CACHE_KEY_FPFILE) {
         item->key = estrndup((char*)slot->key.data.fpfile.fullpath, slot->key.data.fpfile.fullpath_len);
         item->key_len = slot->key.data.fpfile.fullpath_len;
+        item->filename_key = item->key;
     } else {
-        apc_eprint("Internal error, invalid entry type.");
+        apc_error("Internal error, invalid entry type." TSRMLS_CC);
     }
 
     ALLOC_INIT_ZVAL(item->value);
@@ -76,26 +81,38 @@ static apc_iterator_item_t* apc_iterator_item_ctor(apc_iterator_t *iterator, slo
     }
     if (APC_ITER_DEVICE & iterator->format) {
         if(slot->key.type == APC_CACHE_KEY_FILE) {
+#ifdef PHP_WIN32
+			char buf[20];
+			sprintf(buf, "%I64d", slot->key.data.file.device);
+			add_assoc_string(item->value, "device", buf, 1);
+#else
             add_assoc_long(item->value, "device", slot->key.data.file.device);
+#endif
         }
     }
     if (APC_ITER_INODE & iterator->format) {
         if(slot->key.type == APC_CACHE_KEY_FILE) {
+#ifdef PHP_WIN32
+			char buf[20];
+			sprintf(buf, "%I64d", slot->key.data.file.device);
+			add_assoc_string(item->value, "device", buf, 1);
+#else
             add_assoc_long(item->value, "inode", slot->key.data.file.inode);
+#endif
         }
     }
     if (APC_ITER_KEY & iterator->format) {
-        add_assoc_stringl(item->value, "key", item->key, item->key_len, 1);
+        add_assoc_stringl(item->value, "key", item->key, (item->key_len - 1), 1);
     }
     if (APC_ITER_VALUE & iterator->format) {
         if(slot->value->type == APC_CACHE_ENTRY_USER) {
 
-            ctxt.pool = apc_pool_create(APC_UNPOOL, apc_php_malloc, apc_php_free, NULL, NULL);
+            ctxt.pool = apc_pool_create(APC_UNPOOL, apc_php_malloc, apc_php_free, NULL, NULL TSRMLS_CC);
             ctxt.copy = APC_COPY_OUT_USER;
 
             MAKE_STD_ZVAL(zvalue);
-            apc_cache_fetch_zval(zvalue, slot->value->data.user.val, &ctxt);
-            apc_pool_destroy(ctxt.pool);
+            apc_cache_fetch_zval(zvalue, slot->value->data.user.val, &ctxt TSRMLS_CC);
+            apc_pool_destroy(ctxt.pool TSRMLS_CC);
             add_assoc_zval(item->value, "value", zvalue);
         }
     }
@@ -141,7 +158,7 @@ static apc_iterator_item_t* apc_iterator_item_ctor(apc_iterator_t *iterator, slo
 /* {{{ apc_iterator_clone */
 static zend_object_value apc_iterator_clone(zval *zobject TSRMLS_DC) {
     zend_object_value value = {0};
-    apc_eprint("APCIterator object cannot be cloned.");
+    apc_error("APCIterator object cannot be cloned." TSRMLS_CC);
     return value;
 }
 /* }}} */
@@ -216,11 +233,11 @@ static int apc_iterator_search_match(apc_iterator_t *iterator, slot_t **slot) {
     char *key;
     int key_len;
     char *fname_key = NULL;
-    int fname_key_len;
+    int fname_key_len = 0;
     int rval = 1;
 
     if ((*slot)->key.type == APC_CACHE_KEY_FILE) {
-        key = estrdup((*slot)->value->data.file.filename);
+        key = (*slot)->value->data.file.filename;
         key_len = strlen(key);
         fname_key_len = spprintf(&fname_key, 0, "%ld %ld", (*slot)->key.data.file.device, (*slot)->key.data.file.inode);
     } else if ((*slot)->key.type == APC_CACHE_KEY_USER) {
@@ -229,6 +246,8 @@ static int apc_iterator_search_match(apc_iterator_t *iterator, slot_t **slot) {
     } else if ((*slot)->key.type == APC_CACHE_KEY_FPFILE) {
         key = (char*)(*slot)->key.data.fpfile.fullpath;
         key_len = (*slot)->key.data.fpfile.fullpath_len;
+    } else {
+        return 0;
     }
 
 #ifdef ITERATOR_PCRE
@@ -238,7 +257,7 @@ static int apc_iterator_search_match(apc_iterator_t *iterator, slot_t **slot) {
 #endif
             
     if (iterator->search_hash) {
-        rval = zend_hash_exists(iterator->search_hash, key, key_len+1);
+        rval = zend_hash_exists(iterator->search_hash, key, key_len);
         if (!rval && fname_key) {
             rval = zend_hash_exists(iterator->search_hash, fname_key, fname_key_len+1);
         }
@@ -249,7 +268,7 @@ static int apc_iterator_search_match(apc_iterator_t *iterator, slot_t **slot) {
 /* }}} */
 
 /* {{{ apc_iterator_fetch_active */
-static int apc_iterator_fetch_active(apc_iterator_t *iterator) {
+static int apc_iterator_fetch_active(apc_iterator_t *iterator TSRMLS_DC) {
     int count=0;
     slot_t **slot;
     apc_iterator_item_t *item;
@@ -264,9 +283,9 @@ static int apc_iterator_fetch_active(apc_iterator_t *iterator) {
         while(*slot) {
             if (apc_iterator_search_match(iterator, slot)) {
                 count++;
-                item = apc_iterator_item_ctor(iterator, slot);
+                item = apc_iterator_item_ctor(iterator, slot TSRMLS_CC);
                 if (item) {
-                    apc_stack_push(iterator->stack, item);
+                    apc_stack_push(iterator->stack, item TSRMLS_CC);
                 }
             }
             slot = &(*slot)->next;
@@ -280,7 +299,7 @@ static int apc_iterator_fetch_active(apc_iterator_t *iterator) {
 /* }}} */
 
 /* {{{ apc_iterator_fetch_deleted */
-static int apc_iterator_fetch_deleted(apc_iterator_t *iterator) {
+static int apc_iterator_fetch_deleted(apc_iterator_t *iterator TSRMLS_DC) {
     int count=0;
     slot_t **slot;
     apc_iterator_item_t *item;
@@ -295,9 +314,9 @@ static int apc_iterator_fetch_deleted(apc_iterator_t *iterator) {
     while ((*slot) && count < iterator->chunk_size) {
         if (apc_iterator_search_match(iterator, slot)) {
             count++;
-            item = apc_iterator_item_ctor(iterator, slot);
+            item = apc_iterator_item_ctor(iterator, slot TSRMLS_CC);
             if (item) {
-                apc_stack_push(iterator->stack, item);
+                apc_stack_push(iterator->stack, item TSRMLS_CC);
             }
         }
         slot = &(*slot)->next;
@@ -310,7 +329,7 @@ static int apc_iterator_fetch_deleted(apc_iterator_t *iterator) {
 /* }}} */
 
 /* {{{ apc_iterator_totals */
-static void apc_iterator_totals(apc_iterator_t *iterator) {
+static void apc_iterator_totals(apc_iterator_t *iterator TSRMLS_DC) {
     slot_t **slot;
     int i;
 
@@ -346,13 +365,17 @@ PHP_METHOD(apc_iterator, __construct) {
         return;
     }
 
+    if (!APCG(enabled)) {
+        apc_error("APC must be enabled to use APCIterator." TSRMLS_CC);
+    }
+
     if (chunk_size < 0) {
-        apc_eprint("APCIterator chunk size must be greater than 0.");
+        apc_error("APCIterator chunk size must be 0 or greater." TSRMLS_CC);
         return;
     }
 
     if (format > APC_ITER_ALL) {
-        apc_eprint("APCIterator format is invalid.");
+        apc_error("APCIterator format is invalid." TSRMLS_CC);
         return;
     }
 
@@ -361,7 +384,7 @@ PHP_METHOD(apc_iterator, __construct) {
     } else if (list == APC_LIST_DELETED) {
         iterator->fetch = apc_iterator_fetch_deleted;
     } else {
-        apc_wprint("APCIterator invalid list type.");
+        apc_warning("APCIterator invalid list type." TSRMLS_CC);
         return;
     }
 
@@ -375,7 +398,7 @@ PHP_METHOD(apc_iterator, __construct) {
     iterator->stack_idx = 0;
     iterator->key_idx = 0;
     iterator->chunk_size = chunk_size == 0 ? APC_DEFAULT_CHUNK_SIZE : chunk_size;
-    iterator->stack = apc_stack_create(chunk_size);
+    iterator->stack = apc_stack_create(chunk_size TSRMLS_CC);
     iterator->format = format;
     iterator->totals_flag = 0;
     iterator->count = 0;
@@ -391,10 +414,10 @@ PHP_METHOD(apc_iterator, __construct) {
         iterator->re = pcre_get_compiled_regex(Z_STRVAL_P(search), NULL, NULL TSRMLS_CC);
 
         if(!iterator->re) {
-            apc_eprint("Could not compile regular expression: %s", Z_STRVAL_P(search));
+            apc_error("Could not compile regular expression: %s" TSRMLS_CC, Z_STRVAL_P(search));
         }
 #else
-        apc_eprint("Regular expressions support is not enabled, please enable PCRE for APCIterator regex support");
+        apc_error("Regular expressions support is not enabled, please enable PCRE for APCIterator regex support" TSRMLS_CC);
 #endif
     } else if (search && Z_TYPE_P(search) == IS_ARRAY) {
         Z_ADDREF_P(search);
@@ -406,8 +429,11 @@ PHP_METHOD(apc_iterator, __construct) {
 
 /* {{{ proto APCIterator::rewind() */
 PHP_METHOD(apc_iterator, rewind) {
-    zval *object = getThis();
-    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(object TSRMLS_CC);
+    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(getThis() TSRMLS_CC);
+
+    if (zend_parse_parameters_none() == FAILURE) {
+        return;
+    }
 
     if (iterator->initialized == 0) {
         RETURN_FALSE;
@@ -416,21 +442,24 @@ PHP_METHOD(apc_iterator, rewind) {
     iterator->slot_idx = 0;
     iterator->stack_idx = 0;
     iterator->key_idx = 0;
-    iterator->fetch(iterator);
+    iterator->fetch(iterator TSRMLS_CC);
 }
 /* }}} */
 
 /* {{{ proto boolean APCIterator::valid() */
 PHP_METHOD(apc_iterator, valid) {
-    zval *object = getThis();
-    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(object TSRMLS_CC);
+    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(getThis() TSRMLS_CC);
+
+    if (zend_parse_parameters_none() == FAILURE) {
+        return;
+    }
 
     if (iterator->initialized == 0) {
         RETURN_FALSE;
     }
 
     if (apc_stack_size(iterator->stack) == iterator->stack_idx) {
-        iterator->fetch(iterator);
+        iterator->fetch(iterator TSRMLS_CC);
     }
 
     RETURN_BOOL(apc_stack_size(iterator->stack) == 0 ? 0 : 1);
@@ -439,17 +468,23 @@ PHP_METHOD(apc_iterator, valid) {
 
 /* {{{ proto mixed APCIterator::current() */
 PHP_METHOD(apc_iterator, current) {
-    zval *object = getThis();
     apc_iterator_item_t *item;
-    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(object TSRMLS_CC);
+    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(getThis() TSRMLS_CC);
+
+    if (zend_parse_parameters_none() == FAILURE) {
+        return;
+    }
+
     if (iterator->initialized == 0) {
         RETURN_FALSE;
     }
+
     if (apc_stack_size(iterator->stack) == iterator->stack_idx) {
-        if (iterator->fetch(iterator) == 0) {
+        if (iterator->fetch(iterator TSRMLS_CC) == 0) {
             RETURN_FALSE;
         }
     }
+
     item = apc_stack_get(iterator->stack, iterator->stack_idx);
     RETURN_ZVAL(item->value, 1, 0);
 }
@@ -457,20 +492,27 @@ PHP_METHOD(apc_iterator, current) {
 
 /* {{{ proto string APCIterator::key() */
 PHP_METHOD(apc_iterator, key) {
-    zval *object = getThis();
     apc_iterator_item_t *item;
-    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(object TSRMLS_CC);
+    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(getThis() TSRMLS_CC);
+
+    if (zend_parse_parameters_none() == FAILURE) {
+        return;
+    }
+
     if (iterator->initialized == 0 || apc_stack_size(iterator->stack) == 0) {
         RETURN_FALSE;
     }
+
     if (apc_stack_size(iterator->stack) == iterator->stack_idx) {
-        if (iterator->fetch(iterator) == 0) {
+        if (iterator->fetch(iterator TSRMLS_CC) == 0) {
             RETURN_FALSE;
         }
     }
+
     item = apc_stack_get(iterator->stack, iterator->stack_idx);
+
     if (item->key) {
-        RETURN_STRINGL(item->key, item->key_len, 1);
+        RETURN_STRINGL(item->key, (item->key_len-1), 1);
     } else {
         RETURN_LONG(iterator->key_idx);
     }
@@ -479,45 +521,57 @@ PHP_METHOD(apc_iterator, key) {
 
 /* {{{ proto APCIterator::next() */
 PHP_METHOD(apc_iterator, next) {
-    zval *object = getThis();
-    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(object TSRMLS_CC);
+    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(getThis() TSRMLS_CC);
+
+    if (zend_parse_parameters_none() == FAILURE) {
+        return;
+    }
+
     if (iterator->initialized == 0 || apc_stack_size(iterator->stack) == 0) {
         RETURN_FALSE;
     }
+
     iterator->stack_idx++;
     iterator->key_idx++;
+
     RETURN_TRUE;
 }
 /* }}} */
 
 /* {{{ proto long APCIterator::getTotalHits() */
 PHP_METHOD(apc_iterator, getTotalHits) {
-    zval *object = getThis();
-    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(object TSRMLS_CC);
+    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(getThis() TSRMLS_CC);
+
+    if (zend_parse_parameters_none() == FAILURE) {
+        return;
+    }
 
     if (iterator->initialized == 0) {
         RETURN_FALSE;
     }
 
     if (iterator->totals_flag == 0) {
-        apc_iterator_totals(iterator);
+        apc_iterator_totals(iterator TSRMLS_CC);
     }
 
     RETURN_LONG(iterator->hits);
 }
 /* }}} */
 
-/* {{{ proto long APCIterator:;getTotalSize() */
+/* {{{ proto long APCIterator::getTotalSize() */
 PHP_METHOD(apc_iterator, getTotalSize) {
-    zval *object = getThis();
-    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(object TSRMLS_CC);
+    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(getThis() TSRMLS_CC);
+
+    if (zend_parse_parameters_none() == FAILURE) {
+        return;
+    }
 
     if (iterator->initialized == 0) {
         RETURN_FALSE;
     }
 
     if (iterator->totals_flag == 0) {
-        apc_iterator_totals(iterator);
+        apc_iterator_totals(iterator TSRMLS_CC);
     }
 
     RETURN_LONG(iterator->size);
@@ -526,32 +580,56 @@ PHP_METHOD(apc_iterator, getTotalSize) {
 
 /* {{{ proto long APCIterator::getTotalCount() */
 PHP_METHOD(apc_iterator, getTotalCount) {
-    zval *object = getThis();
-    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(object TSRMLS_CC);
+    apc_iterator_t *iterator = (apc_iterator_t*)zend_object_store_get_object(getThis() TSRMLS_CC);
+
+    if (zend_parse_parameters_none() == FAILURE) {
+        return;
+    }
 
     if (iterator->initialized == 0) {
         RETURN_FALSE;
     }
 
     if (iterator->totals_flag == 0) {
-        apc_iterator_totals(iterator);
+        apc_iterator_totals(iterator TSRMLS_CC);
     }
 
     RETURN_LONG(iterator->count);
 }
 /* }}} */
 
+/* {{{ arginfo */
+#if (PHP_MAJOR_VERSION >= 6 || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3))
+# define PHP_APC_ARGINFO
+#else
+# define PHP_APC_ARGINFO static
+#endif
+
+PHP_APC_ARGINFO
+ZEND_BEGIN_ARG_INFO_EX(arginfo_apc_iterator___construct, 0, 0, 1)
+	ZEND_ARG_INFO(0, cache)
+	ZEND_ARG_INFO(0, search)
+	ZEND_ARG_INFO(0, format)
+	ZEND_ARG_INFO(0, chunk_size)
+	ZEND_ARG_INFO(0, list)
+ZEND_END_ARG_INFO()
+
+PHP_APC_ARGINFO
+ZEND_BEGIN_ARG_INFO_EX(arginfo_apc_iterator_void, 0, 0, 0)
+ZEND_END_ARG_INFO()
+/* }}} */
+
 /* {{{ apc_iterator_functions */
-static function_entry apc_iterator_functions[] = {
-    PHP_ME(apc_iterator, __construct, NULL, ZEND_ACC_PUBLIC|ZEND_ACC_CTOR)
-    PHP_ME(apc_iterator, rewind, NULL, ZEND_ACC_PUBLIC)
-    PHP_ME(apc_iterator, current, NULL, ZEND_ACC_PUBLIC)
-    PHP_ME(apc_iterator, key, NULL, ZEND_ACC_PUBLIC)
-    PHP_ME(apc_iterator, next, NULL, ZEND_ACC_PUBLIC)
-    PHP_ME(apc_iterator, valid, NULL, ZEND_ACC_PUBLIC)
-    PHP_ME(apc_iterator, getTotalHits, NULL, ZEND_ACC_PUBLIC)
-    PHP_ME(apc_iterator, getTotalSize, NULL, ZEND_ACC_PUBLIC)
-    PHP_ME(apc_iterator, getTotalCount, NULL, ZEND_ACC_PUBLIC)
+static zend_function_entry apc_iterator_functions[] = {
+    PHP_ME(apc_iterator, __construct, arginfo_apc_iterator___construct, ZEND_ACC_PUBLIC|ZEND_ACC_CTOR)
+    PHP_ME(apc_iterator, rewind, arginfo_apc_iterator_void, ZEND_ACC_PUBLIC)
+    PHP_ME(apc_iterator, current, arginfo_apc_iterator_void, ZEND_ACC_PUBLIC)
+    PHP_ME(apc_iterator, key, arginfo_apc_iterator_void, ZEND_ACC_PUBLIC)
+    PHP_ME(apc_iterator, next, arginfo_apc_iterator_void, ZEND_ACC_PUBLIC)
+    PHP_ME(apc_iterator, valid, arginfo_apc_iterator_void, ZEND_ACC_PUBLIC)
+    PHP_ME(apc_iterator, getTotalHits, arginfo_apc_iterator_void, ZEND_ACC_PUBLIC)
+    PHP_ME(apc_iterator, getTotalSize, arginfo_apc_iterator_void, ZEND_ACC_PUBLIC)
+    PHP_ME(apc_iterator, getTotalCount, arginfo_apc_iterator_void, ZEND_ACC_PUBLIC)
     {NULL, NULL, NULL}
 };
 /* }}} */
@@ -600,7 +678,7 @@ int apc_iterator_delete(zval *zobj TSRMLS_DC) {
     apc_iterator_item_t *item;
 
     if (!ce || !instanceof_function(ce, apc_iterator_ce TSRMLS_CC)) {
-        apc_eprint("apc_delete object argument must be instance of APCIterator");
+        apc_error("apc_delete object argument must be instance of APCIterator" TSRMLS_CC);
         return 0;
     }
     iterator = (apc_iterator_t*)zend_object_store_get_object(zobj TSRMLS_CC);
@@ -609,13 +687,13 @@ int apc_iterator_delete(zval *zobj TSRMLS_DC) {
         return 0;
     }
 
-    while (iterator->fetch(iterator)) {
+    while (iterator->fetch(iterator TSRMLS_CC)) {
         while (iterator->stack_idx < apc_stack_size(iterator->stack)) {
             item = apc_stack_get(iterator->stack, iterator->stack_idx++);
             if (iterator->cache == apc_cache) {
-                apc_cache_delete(apc_cache, item->filename_key, strlen(item->filename_key)+1);
+                apc_cache_delete(apc_cache, item->filename_key, strlen(item->filename_key) + 1 TSRMLS_CC);
             } else {
-                apc_cache_user_delete(apc_user_cache, item->key, item->key_len+1);
+                apc_cache_user_delete(apc_user_cache, item->key, item->key_len TSRMLS_CC);
             }
         }
     }
