@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2008 The PHP Group                                     |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -25,7 +25,7 @@
 
  */
 
-/* $Id: apc_pool.c 281353 2009-05-28 23:45:08Z kalle $ */
+/* $Id: apc_pool.c 307328 2011-01-10 06:21:53Z gopalv $ */
 
 
 #include "apc_pool.h"
@@ -37,8 +37,8 @@
 
 
 /* {{{ forward references */
-static apc_pool* apc_unpool_create(apc_pool_type type, apc_malloc_t, apc_free_t, apc_protect_t, apc_unprotect_t);
-static apc_pool* apc_realpool_create(apc_pool_type type, apc_malloc_t, apc_free_t, apc_protect_t, apc_unprotect_t);
+static apc_pool* apc_unpool_create(apc_pool_type type, apc_malloc_t, apc_free_t, apc_protect_t, apc_unprotect_t TSRMLS_DC);
+static apc_pool* apc_realpool_create(apc_pool_type type, apc_malloc_t, apc_free_t, apc_protect_t, apc_unprotect_t TSRMLS_DC);
 /* }}} */
 
 /* {{{ apc_pool_create */
@@ -46,26 +46,27 @@ apc_pool* apc_pool_create(apc_pool_type pool_type,
                             apc_malloc_t allocate, 
                             apc_free_t deallocate,
                             apc_protect_t protect,
-                            apc_unprotect_t unprotect)
+                            apc_unprotect_t unprotect
+			    TSRMLS_DC)
 {
     if(pool_type == APC_UNPOOL) {
         return apc_unpool_create(pool_type, allocate, deallocate,
-                                            protect, unprotect );
+                                            protect, unprotect TSRMLS_CC);
     }
 
     return apc_realpool_create(pool_type, allocate, deallocate, 
-                                          protect,  unprotect );
+                                          protect,  unprotect TSRMLS_CC);
 }
 /* }}} */
 
 /* {{{ apc_pool_destroy */
-void apc_pool_destroy(apc_pool *pool)
+void apc_pool_destroy(apc_pool *pool TSRMLS_DC)
 {
     apc_free_t deallocate = pool->deallocate;
     apc_pcleanup_t cleanup = pool->cleanup;
 
-    cleanup(pool);
-    deallocate(pool);
+    cleanup(pool TSRMLS_CC);
+    deallocate(pool TSRMLS_CC);
 }
 /* }}} */
 
@@ -78,7 +79,7 @@ struct _apc_unpool {
     /* apc_unpool is a lie! */
 };
 
-static void* apc_unpool_alloc(apc_pool* pool, size_t size) 
+static void* apc_unpool_alloc(apc_pool* pool, size_t size TSRMLS_DC) 
 {
     apc_unpool *upool = (apc_unpool*)pool;
 
@@ -87,27 +88,28 @@ static void* apc_unpool_alloc(apc_pool* pool, size_t size)
     upool->parent.size += size;
     upool->parent.used += size;
 
-    return allocate(size);
+    return allocate(size TSRMLS_CC);
 }
 
-static void apc_unpool_free(apc_pool* pool, void *ptr)
+static void apc_unpool_free(apc_pool* pool, void *ptr TSRMLS_DC)
 {
     apc_unpool *upool = (apc_unpool*) pool;
 
     apc_free_t deallocate = upool->parent.deallocate;
 
-    deallocate(ptr);
+    deallocate(ptr TSRMLS_CC);
 }
 
-static void apc_unpool_cleanup(apc_pool* pool)
+static void apc_unpool_cleanup(apc_pool* pool TSRMLS_DC)
 {
 }
 
 static apc_pool* apc_unpool_create(apc_pool_type type, 
                     apc_malloc_t allocate, apc_free_t deallocate,
-                    apc_protect_t protect, apc_unprotect_t unprotect)
+                    apc_protect_t protect, apc_unprotect_t unprotect
+		    TSRMLS_DC)
 {
-    apc_unpool* upool = allocate(sizeof(apc_unpool));
+    apc_unpool* upool = allocate(sizeof(apc_unpool) TSRMLS_CC);
 
     if (!upool) {
         return NULL;
@@ -166,6 +168,8 @@ struct _apc_realpool
     size_t     dsize;
     void       *owner;
 
+    unsigned long count;
+
     pool_block *head;
     pool_block first; 
 };
@@ -204,13 +208,13 @@ static const unsigned char decaff[] =  {
 } while(0)
 
 /* {{{ create_pool_block */
-static pool_block* create_pool_block(apc_realpool *rpool, size_t size)
+static pool_block* create_pool_block(apc_realpool *rpool, size_t size TSRMLS_DC)
 {
     apc_malloc_t allocate = rpool->parent.allocate;
 
     size_t realsize = sizeof(pool_block) + ALIGNWORD(size);
 
-    pool_block* entry = allocate(realsize);
+    pool_block* entry = allocate(realsize TSRMLS_CC);
 
     if (!entry) {
         return NULL;
@@ -220,12 +224,14 @@ static pool_block* create_pool_block(apc_realpool *rpool, size_t size)
     
     rpool->parent.size += realsize;
 
+    rpool->count++;
+
     return entry;
 }
 /* }}} */
 
 /* {{{ apc_realpool_alloc */
-static void* apc_realpool_alloc(apc_pool *pool, size_t size)
+static void* apc_realpool_alloc(apc_pool *pool, size_t size TSRMLS_DC)
 {
     apc_realpool *rpool = (apc_realpool*)pool;
     unsigned char *p = NULL;
@@ -234,9 +240,9 @@ static void* apc_realpool_alloc(apc_pool *pool, size_t size)
     unsigned char *redzone  = NULL;
     size_t redsize  = 0;
     size_t *sizeinfo= NULL;
-
-    pool_block *entry;
-
+    pool_block *entry = NULL;
+    unsigned long i;
+    
     if(APC_POOL_HAS_REDZONES(pool)) {
         redsize = REDZONE_SIZE(size); /* redsize might be re-using word size padding */
         realsize = size + redsize;    /* recalculating realsize */
@@ -248,8 +254,16 @@ static void* apc_realpool_alloc(apc_pool *pool, size_t size)
         realsize += ALIGNWORD(sizeof(size_t));
     }
 
+    /* upgrade the pool type to reduce overhead */
+    if(rpool->count > 4 && rpool->dsize < 4096) {
+        rpool->dsize = 4096;
+    } else if(rpool->count > 8 && rpool->dsize < 8192) {
+        rpool->dsize = 8192;
+    }
 
-    for(entry = rpool->head; entry != NULL; entry = entry->next) {
+    /* minimize look-back, a value of 8 seems to give similar fill-ratios (+2%)
+     * as looping through the entire list. And much faster in allocations. */
+    for(entry = rpool->head, i = 0; entry != NULL && (i < 8); entry = entry->next, i++) {
         if(entry->avail >= realsize) {
             goto found;
         }
@@ -257,7 +271,7 @@ static void* apc_realpool_alloc(apc_pool *pool, size_t size)
 
     poolsize = ALIGNSIZE(realsize, rpool->dsize);
 
-    entry = create_pool_block(rpool, poolsize);
+    entry = create_pool_block(rpool, poolsize TSRMLS_CC);
 
     if(!entry) {
         return NULL;
@@ -302,8 +316,12 @@ found:
  * Checking integrity at runtime, does an
  * overwrite check only when the sizeinfo
  * is set.
+ *
+ * Marked as used in gcc, so that this function
+ * is accessible from gdb, eventhough it is never
+ * used in code in non-debug builds.
  */
-static int apc_realpool_check_integrity(apc_realpool *rpool) 
+static APC_USED int apc_realpool_check_integrity(apc_realpool *rpool) 
 {
     apc_pool *pool = &(rpool->parent); 
     pool_block *entry;
@@ -322,6 +340,7 @@ static int apc_realpool_check_integrity(apc_realpool *rpool)
 
     if(!APC_POOL_HAS_REDZONES(pool) ||
         !APC_POOL_HAS_SIZEINFO(pool)) {
+        (void)pool; /* remove unused warning */
         return 1;
     }
 
@@ -358,18 +377,18 @@ static int apc_realpool_check_integrity(apc_realpool *rpool)
 }
 /* }}} */
 
-/* {{{ apc_pool_free */
+/* {{{ apc_realpool_free */
 /*
  * free does not do anything other than
  * check for redzone values when free'ing
  * data areas.
  */
-static void apc_realpool_free(apc_pool *pool, void *p)
+static void apc_realpool_free(apc_pool *pool, void *p TSRMLS_DC)
 {
 }
 /* }}} */
 
-static void apc_realpool_cleanup(apc_pool *pool) 
+static void apc_realpool_cleanup(apc_pool *pool TSRMLS_DC) 
 {
     pool_block *entry;
     pool_block *tmp;
@@ -382,14 +401,15 @@ static void apc_realpool_cleanup(apc_pool *pool)
 
     while(entry->next != NULL) {
         tmp = entry->next;
-        deallocate(entry);
+        deallocate(entry TSRMLS_CC);
         entry = tmp;
     }
 }
 
 /* {{{ apc_realpool_create */
 static apc_pool* apc_realpool_create(apc_pool_type type, apc_malloc_t allocate, apc_free_t deallocate, 
-                                                         apc_protect_t protect, apc_unprotect_t unprotect)
+                                                         apc_protect_t protect, apc_unprotect_t unprotect
+                                                         TSRMLS_DC)
 {
 
     size_t dsize = 0;
@@ -412,7 +432,7 @@ static apc_pool* apc_realpool_create(apc_pool_type type, apc_malloc_t allocate,
             return NULL;
     }
 
-    rpool = (apc_realpool*)allocate(sizeof(apc_realpool) + ALIGNWORD(dsize));
+    rpool = (apc_realpool*)allocate((sizeof(apc_realpool) + ALIGNWORD(dsize)) TSRMLS_CC);
 
     if(!rpool) {
         return NULL;
@@ -435,6 +455,7 @@ static apc_pool* apc_realpool_create(apc_pool_type type, apc_malloc_t allocate,
 
     rpool->dsize = dsize;
     rpool->head = NULL;
+    rpool->count = 0;
 
     INIT_POOL_BLOCK(rpool, &(rpool->first), dsize);
 
@@ -450,19 +471,21 @@ void apc_pool_init()
     /* put all ye sanity checks here */
     assert(sizeof(decaff) > REDZONE_SIZE(ALIGNWORD(sizeof(char))));
     assert(sizeof(pool_block) == ALIGNWORD(sizeof(pool_block)));
+#if APC_POOL_DEBUG
     assert((APC_POOL_SIZE_MASK & (APC_POOL_SIZEINFO | APC_POOL_REDZONES)) == 0);
+#endif
 }
 /* }}} */
 
 /* {{{ apc_pstrdup */
-void* apc_pstrdup(const char* s, apc_pool* pool)
+void* APC_ALLOC apc_pstrdup(const char* s, apc_pool* pool TSRMLS_DC)
 {
-    return s != NULL ? apc_pmemcpy(s, strlen(s)+1, pool) : NULL;
+    return s != NULL ? apc_pmemcpy(s, (strlen(s) + 1), pool TSRMLS_CC) : NULL;
 }
 /* }}} */
 
 /* {{{ apc_pmemcpy */
-void* apc_pmemcpy(const void* p, size_t n, apc_pool* pool)
+void* APC_ALLOC apc_pmemcpy(const void* p, size_t n, apc_pool* pool TSRMLS_DC)
 {
     void* q;
 
