@@ -2,7 +2,7 @@
   +----------------------------------------------------------------------+
   | APC                                                                  |
   +----------------------------------------------------------------------+
-  | Copyright (c) 2006-2008 The PHP Group                                |
+  | Copyright (c) 2006-2011 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 3.01 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
@@ -26,7 +26,7 @@
 
  */
 
-/* $Id: apc_sma.c 284580 2009-07-22 06:05:31Z kalle $ */
+/* $Id: apc_sma.c 307264 2011-01-08 13:20:20Z gopalv $ */
 
 #include "apc_sma.h"
 #include "apc.h"
@@ -124,26 +124,26 @@ struct block_t {
 /* {{{ sma_debug_state(apc_sma_segment_t *segment, int canary_check, int verbose)
  *        useful for debuging state of memory blocks and free list, and sanity checking
  */
-static void sma_debug_state(void* shmaddr, int canary_check, int verbose) {
+static void sma_debug_state(void* shmaddr, int canary_check, int verbose TSRMLS_DC) {
     sma_header_t *header = (sma_header_t*)shmaddr;
     block_t *cur = BLOCKAT(ALIGNWORD(sizeof(sma_header_t)));
     block_t *prv = NULL;
     size_t avail;
 
     /* Verify free list */
-    if (verbose) apc_wprint("Free List: ");
+    if (verbose) apc_warning("Free List: " TSRMLS_CC);
     while(1) {
-        if (verbose) apc_wprint(" 0x%x[%d] (s%d)", cur, OFFSET(cur), cur->size);
+        if (verbose) apc_warning(" 0x%x[%d] (s%d)" TSRMLS_CC, cur, OFFSET(cur), cur->size);
         if (canary_check) CHECK_CANARY(cur);
         if (!cur->fnext) break;
         cur = BLOCKAT(cur->fnext);
         avail += cur->size;
         if (prv == cur) {
-            apc_wprint("Circular list detected!");
+            apc_warning("Circular list detected!" TSRMLS_CC);
             assert(0);
         }
         if (prv && cur->fprev != OFFSET(prv)) {
-            apc_wprint("Previous pointer does not point to previous!");
+            apc_warning("Previous pointer does not point to previous!" TSRMLS_CC);
             assert(0);
         }
         prv = cur;
@@ -151,13 +151,13 @@ static void sma_debug_state(void* shmaddr, int canary_check, int verbose) {
     assert(avail == header->avail);
 
     /* Verify each block */
-    if (verbose) apc_wprint("Block List: ");
+    if (verbose) apc_warning("Block List: " TSRMLS_CC);
     cur = BLOCKAT(ALIGNWORD(sizeof(sma_header_t)));
     while(1) {
         if(!cur->fnext) {
-            if (verbose) apc_wprint(" 0x%x[%d] (s%d) (u)", cur, OFFSET(cur), cur->size);
+            if (verbose) apc_warning(" 0x%x[%d] (s%d) (u)" TSRMLS_CC, cur, OFFSET(cur), cur->size);
         } else {
-            if (verbose) apc_wprint(" 0x%x[%d] (s%d) (f)", cur, OFFSET(cur), cur->size);
+            if (verbose) apc_warning(" 0x%x[%d] (s%d) (f)" TSRMLS_CC, cur, OFFSET(cur), cur->size);
         }
         if (canary_check) CHECK_CANARY(cur);
         if (!cur->size && !cur->fnext) break;
@@ -167,7 +167,7 @@ static void sma_debug_state(void* shmaddr, int canary_check, int verbose) {
             cur = NEXT_SBLOCK(cur);
         }
         if (prv == cur) {
-            apc_wprint("Circular list detected!");
+            apc_warning("Circular list detected!" TSRMLS_CC);
             assert(0);
         }
         prv = cur;
@@ -177,7 +177,7 @@ static void sma_debug_state(void* shmaddr, int canary_check, int verbose) {
 #endif
 
 /* {{{ sma_allocate: tries to allocate at least size bytes in a segment */
-static size_t sma_allocate(sma_header_t* header, size_t size, size_t fragment, size_t *allocated)
+static APC_HOTSPOT size_t sma_allocate(sma_header_t* header, size_t size, size_t fragment, size_t *allocated)
 {
     void* shmaddr;          /* header of shared memory segment */
     block_t* prv;           /* block prior to working block */
@@ -274,7 +274,7 @@ static size_t sma_allocate(sma_header_t* header, size_t size, size_t fragment, s
 /* }}} */
 
 /* {{{ sma_deallocate: deallocates the block at the given offset */
-static size_t sma_deallocate(void* shmaddr, size_t offset)
+static APC_HOTSPOT size_t sma_deallocate(void* shmaddr, size_t offset)
 {
     sma_header_t* header;   /* header of shared memory segment */
     block_t* cur;       /* the new block to insert */
@@ -333,7 +333,7 @@ static size_t sma_deallocate(void* shmaddr, size_t offset)
 
 /* {{{ apc_sma_init */
 
-void apc_sma_init(int numseg, size_t segsize, char *mmap_file_mask)
+void apc_sma_init(int numseg, size_t segsize, char *mmap_file_mask TSRMLS_DC)
 {
     uint i;
 
@@ -360,18 +360,18 @@ void apc_sma_init(int numseg, size_t segsize, char *mmap_file_mask)
 
     sma_segsize = segsize > 0 ? segsize : DEFAULT_SEGSIZE;
 
-    sma_segments = (apc_segment_t*) apc_emalloc(sma_numseg*sizeof(apc_segment_t));
+    sma_segments = (apc_segment_t*) apc_emalloc((sma_numseg * sizeof(apc_segment_t)) TSRMLS_CC);
 
     for (i = 0; i < sma_numseg; i++) {
         sma_header_t*   header;
-        block_t*    block;
+        block_t     *first, *empty, *last;
         void*       shmaddr;
 
 #if APC_MMAP
-        sma_segments[i] = apc_mmap(mmap_file_mask, sma_segsize);
+        sma_segments[i] = apc_mmap(mmap_file_mask, sma_segsize TSRMLS_CC);
         if(sma_numseg != 1) memcpy(&mmap_file_mask[strlen(mmap_file_mask)-6], "XXXXXX", 6);
 #else
-        sma_segments[i] = apc_shm_attach(apc_shm_create(NULL, i, sma_segsize));
+        sma_segments[i] = apc_shm_attach(apc_shm_create(i, sma_segsize TSRMLS_CC), sma_segsize TSRMLS_CC);
 #endif
         
         sma_segments[i].size = sma_segsize;
@@ -388,39 +388,39 @@ void apc_sma_init(int numseg, size_t segsize, char *mmap_file_mask)
            for(j=0; j<30; j++) header->adist[j] = 0;
         }
 #endif
-        block = BLOCKAT(ALIGNWORD(sizeof(sma_header_t)));
-        block->size = 0;
-        block->fnext = ALIGNWORD(sizeof(sma_header_t)) + ALIGNWORD(sizeof(block_t));
-        block->fprev = 0;
-        block->prev_size = 0;
-        SET_CANARY(block);
+        first = BLOCKAT(ALIGNWORD(sizeof(sma_header_t)));
+        first->size = 0;
+        first->fnext = ALIGNWORD(sizeof(sma_header_t)) + ALIGNWORD(sizeof(block_t));
+        first->fprev = 0;
+        first->prev_size = 0;
+        SET_CANARY(first);
 #ifdef __APC_SMA_DEBUG__
         block->id = -1;
 #endif
-        block = BLOCKAT(block->fnext);
-        block->size = header->avail - ALIGNWORD(sizeof(block_t));
-        block->fnext = OFFSET(block) + block->size;
-        block->fprev = ALIGNWORD(sizeof(sma_header_t));
-        block->prev_size = 0;
-        SET_CANARY(block);
+        empty = BLOCKAT(first->fnext);
+        empty->size = header->avail - ALIGNWORD(sizeof(block_t));
+        empty->fnext = OFFSET(empty) + empty->size;
+        empty->fprev = ALIGNWORD(sizeof(sma_header_t));
+        empty->prev_size = 0;
+        SET_CANARY(empty);
 #ifdef __APC_SMA_DEBUG__
-        block->id = -1;
+        empty->id = -1;
 #endif
-        block = BLOCKAT(block->fnext);
-        block->size = 0;
-        block->fnext = 0;
-        block->fprev =  ALIGNWORD(sizeof(sma_header_t)) + ALIGNWORD(sizeof(block_t));
-        block->prev_size = header->avail - ALIGNWORD(sizeof(block_t));
-        SET_CANARY(block);
+        last = BLOCKAT(empty->fnext);
+        last->size = 0;
+        last->fnext = 0;
+        last->fprev =  OFFSET(empty);
+        last->prev_size = empty->size;
+        SET_CANARY(last);
 #ifdef __APC_SMA_DEBUG__
-        block->id = -1;
+        last->id = -1;
 #endif
     }
 }
 /* }}} */
 
 /* {{{ apc_sma_cleanup */
-void apc_sma_cleanup()
+void apc_sma_cleanup(TSRMLS_D)
 {
     uint i;
 
@@ -429,23 +429,24 @@ void apc_sma_cleanup()
     for (i = 0; i < sma_numseg; i++) {
         apc_lck_destroy(SMA_LCK(i));
 #if APC_MMAP
-        apc_unmap(&sma_segments[i]);
+        apc_unmap(&sma_segments[i] TSRMLS_CC);
 #else
-        apc_shm_detach(&sma_segments[i]);
+        apc_shm_detach(&sma_segments[i] TSRMLS_CC);
 #endif
     }
     sma_initialized = 0;
-    apc_efree(sma_segments);
+    apc_efree(sma_segments TSRMLS_CC);
 }
 /* }}} */
 
 /* {{{ apc_sma_malloc_ex */
-void* apc_sma_malloc_ex(size_t n, size_t fragment, size_t* allocated)
+void* apc_sma_malloc_ex(size_t n, size_t fragment, size_t* allocated TSRMLS_DC)
 {
     size_t off;
     uint i;
+    int nuked = 0;
 
-    TSRMLS_FETCH();
+restart:
     assert(sma_initialized);
     LOCK(SMA_LCK(sma_lastseg));
 
@@ -454,7 +455,7 @@ void* apc_sma_malloc_ex(size_t n, size_t fragment, size_t* allocated)
     if(off == -1 && APCG(current_cache)) { 
         /* retry failed allocation after we expunge */
         UNLOCK(SMA_LCK(sma_lastseg));
-        APCG(current_cache)->expunge_cb(APCG(current_cache), n);
+        APCG(current_cache)->expunge_cb(APCG(current_cache), (n+fragment) TSRMLS_CC);
         LOCK(SMA_LCK(sma_lastseg));
         off = sma_allocate(SMA_HDR(sma_lastseg), n, fragment, allocated);
     }
@@ -479,7 +480,7 @@ void* apc_sma_malloc_ex(size_t n, size_t fragment, size_t* allocated)
         if(off == -1 && APCG(current_cache)) { 
             /* retry failed allocation after we expunge */
             UNLOCK(SMA_LCK(i));
-            APCG(current_cache)->expunge_cb(APCG(current_cache), n);
+            APCG(current_cache)->expunge_cb(APCG(current_cache), (n+fragment) TSRMLS_CC);
             LOCK(SMA_LCK(i));
             off = sma_allocate(SMA_HDR(i), n, fragment, allocated);
         }
@@ -495,30 +496,40 @@ void* apc_sma_malloc_ex(size_t n, size_t fragment, size_t* allocated)
         UNLOCK(SMA_LCK(i));
     }
 
+    /* I've tried being nice, but now you're just asking for it */
+    if(!nuked) {
+        apc_cache->expunge_cb(apc_cache, (n+fragment) TSRMLS_CC);
+        apc_user_cache->expunge_cb(apc_user_cache, (n+fragment) TSRMLS_CC);
+        nuked = 1;
+        goto restart;
+    }
+
+    /* now, I've truly and well given up */
+
     return NULL;
 }
 /* }}} */
 
 /* {{{ apc_sma_malloc */
-void* apc_sma_malloc(size_t n)
+void* apc_sma_malloc(size_t n TSRMLS_DC)
 {
     size_t allocated;
-    void *p = apc_sma_malloc_ex(n, MINBLOCKSIZE, &allocated);
+    void *p = apc_sma_malloc_ex(n, MINBLOCKSIZE, &allocated TSRMLS_CC);
 
     return p;
 }
 /* }}} */
 
 /* {{{ apc_sma_realloc */
-void* apc_sma_realloc(void *p, size_t n)
+void* apc_sma_realloc(void *p, size_t n TSRMLS_DC)
 {
-    apc_sma_free(p);
-    return apc_sma_malloc(n);
+    apc_sma_free(p TSRMLS_CC);
+    return apc_sma_malloc(n TSRMLS_CC);
 }
 /* }}} */
 
 /* {{{ apc_sma_strdup */
-char* apc_sma_strdup(const char* s)
+char* apc_sma_strdup(const char* s TSRMLS_DC)
 {
     void* q;
     int len;
@@ -526,7 +537,7 @@ char* apc_sma_strdup(const char* s)
     if(!s) return NULL;
 
     len = strlen(s)+1;
-    q = apc_sma_malloc(len);
+    q = apc_sma_malloc(len TSRMLS_CC);
     if(!q) return NULL;
     memcpy(q, s, len);
     return q;
@@ -534,7 +545,7 @@ char* apc_sma_strdup(const char* s)
 /* }}} */
 
 /* {{{ apc_sma_free */
-void apc_sma_free(void* p)
+void apc_sma_free(void* p TSRMLS_DC)
 {
     uint i;
     size_t offset;
@@ -560,7 +571,7 @@ void apc_sma_free(void* p)
         }
     }
 
-    apc_eprint("apc_sma_free: could not locate address %p", p);
+    apc_error("apc_sma_free: could not locate address %p" TSRMLS_CC, p);
 }
 /* }}} */
 
@@ -568,7 +579,7 @@ void apc_sma_free(void* p)
 /* {{{ */
 void* apc_sma_protect(void *p)
 {
-    int i;
+    unsigned int i = 0;
     size_t offset;
 
     if (p == NULL) {
@@ -597,7 +608,7 @@ void* apc_sma_protect(void *p)
 /* {{{ */
 void* apc_sma_unprotect(void *p)
 {
-    int i;
+    unsigned int i = 0;
     size_t offset;
 
     if (p == NULL) {
@@ -630,7 +641,7 @@ void* apc_sma_unprotect(void *p) { return p; }
 #endif
 
 /* {{{ apc_sma_info */
-apc_sma_info_t* apc_sma_info(zend_bool limited)
+apc_sma_info_t* apc_sma_info(zend_bool limited TSRMLS_DC)
 {
     apc_sma_info_t* info;
     apc_sma_link_t** link;
@@ -642,11 +653,11 @@ apc_sma_info_t* apc_sma_info(zend_bool limited)
         return NULL;
     }
 
-    info = (apc_sma_info_t*) apc_emalloc(sizeof(apc_sma_info_t));
+    info = (apc_sma_info_t*) apc_emalloc(sizeof(apc_sma_info_t) TSRMLS_CC);
     info->num_seg = sma_numseg;
     info->seg_size = sma_segsize - (ALIGNWORD(sizeof(sma_header_t)) + ALIGNWORD(sizeof(block_t)) + ALIGNWORD(sizeof(block_t)));
 
-    info->list = apc_emalloc(info->num_seg * sizeof(apc_sma_link_t*));
+    info->list = apc_emalloc(info->num_seg * sizeof(apc_sma_link_t*) TSRMLS_CC);
     for (i = 0; i < sma_numseg; i++) {
         info->list[i] = NULL;
     }
@@ -668,7 +679,7 @@ apc_sma_info_t* apc_sma_info(zend_bool limited)
             CHECK_CANARY(cur);
 #endif
 
-            *link = apc_emalloc(sizeof(apc_sma_link_t));
+            *link = apc_emalloc(sizeof(apc_sma_link_t) TSRMLS_CC);
             (*link)->size = cur->size;
             (*link)->offset = prv->fnext;
             (*link)->next = NULL;
@@ -690,7 +701,7 @@ apc_sma_info_t* apc_sma_info(zend_bool limited)
 /* }}} */
 
 /* {{{ apc_sma_free_info */
-void apc_sma_free_info(apc_sma_info_t* info)
+void apc_sma_free_info(apc_sma_info_t* info TSRMLS_DC)
 {
     int i;
 
@@ -699,11 +710,11 @@ void apc_sma_free_info(apc_sma_info_t* info)
         while (p) {
             apc_sma_link_t* q = p;
             p = p->next;
-            apc_efree(q);
+            apc_efree(q TSRMLS_CC);
         }
     }
-    apc_efree(info->list);
-    apc_efree(info);
+    apc_efree(info->list TSRMLS_CC);
+    apc_efree(info TSRMLS_CC);
 }
 /* }}} */
 
@@ -721,6 +732,22 @@ size_t apc_sma_get_avail_mem()
 }
 /* }}} */
 
+/* {{{ apc_sma_get_avail_size */
+zend_bool apc_sma_get_avail_size(size_t size)
+{
+    uint i;
+
+    for (i = 0; i < sma_numseg; i++) {
+    	sma_header_t* header = SMA_HDR(i);
+		if (header->avail > size) {
+			return 1;
+		}
+    }
+    return 0;
+}
+/* }}} */
+
+
 #if ALLOC_DISTRIBUTION
 size_t *apc_sma_get_alloc_distribution(void) {
     sma_header_t* header = (sma_header_t*) segment->sma_shmaddr;
